#!/usr/bin/env python
# coding: utf-8

# # Import and visualize dataset

# In[416]:


# Import dataset and visualize it
import pandas as pd

data = pd.read_csv('bachelor.csv')
data.head()


# In[417]:


# Get more info about the data and the variables
data.info()


# In[418]:


# Visualizing all unique values in the different variables
for col in data:
    print(data[col].unique())


# # Clean the data

# In[419]:


# Find out how many blank values are in the 'intensjon' variables
print(data['intensjon_1'].value_counts())
print(data['intensjon_2'].value_counts())
print(data['intensjon_3'].value_counts())


# In[420]:


# We only have one blank value in each of the three variables
# Replace blank values in variables 44-46 with 4 (a scale from 1-7 where 4 is neutral)
# These blank values are not the result of conditional questions
data['intensjon_1'] = data['intensjon_1'].replace([' '], '4')
data['intensjon_2'] = data['intensjon_2'].replace([' '], '4')
data['intensjon_3'] = data['intensjon_3'].replace([' '], '4')

# Convert variables 44-46 to integers
data = data.astype({'intensjon_1':'int'})
data = data.astype({'intensjon_2':'int'})
data = data.astype({'intensjon_3':'int'})


# In[421]:


# How many blank values are in variable 51 and 58?
print(data['inntekt'].value_counts())
print(data['wtp_PL'].value_counts())


# In[422]:


# We only have one blank value
# Blank values in variable 58 changed to average value
# These blank values are not the result of conditional questions
data['inntekt'] = data['inntekt'].replace([' '], 'x')
data['inntekt'] = pd.to_numeric(data['inntekt'], errors='coerce').astype('float64')
data['inntekt'] = data['inntekt'].fillna(data['inntekt'].mean())
data = data.astype({'inntekt':'int'})


# In[423]:


# We have three blank values
# Blank values in variable 51 changed to average value
# These blank values are not the result of conditional questions
data['wtp_PL'] = data['wtp_PL'].replace([' '], 'x')
data['wtp_PL'] = pd.to_numeric(data['wtp_PL'], errors='coerce').astype('float64')
data['wtp_PL'] = data['wtp_PL'].fillna(data['wtp_PL'].mean())
data = data.astype({'wtp_PL':'int'})


# In[424]:


# How many blank values are in variable 15?
print(data['tv_kamper_utl_sett'].value_counts())


# In[425]:


# 84 blank values
# Change the value 9 to 6 in variable 15
# Convert all blank values to "idk" (6)
data['tv_kamper_utl_sett'] = data['tv_kamper_utl_sett'].replace(['9'], '6')
data['tv_kamper_utl_sett'] = data['tv_kamper_utl_sett'].replace([' '], 'x')
data['tv_kamper_utl_sett'] = pd.to_numeric(data['tv_kamper_utl_sett'], errors='coerce').astype('float64')
data['tv_kamper_utl_sett'] = data['tv_kamper_utl_sett'].fillna(data['tv_kamper_utl_sett'].mean())
data = data.astype({'tv_kamper_utl_sett':'int'})


# In[426]:


# Remove variable 59 ('svartid')
data = data.drop(['svartid'], axis = 1)


# In[427]:


# Remove variables 7-14 that follow the first conditional question
data = data.drop(['liga'], axis = 1)
data = data.drop(['fan_utl_1'], axis = 1)
data = data.drop(['fan_utl_2'], axis = 1)
data = data.drop(['fan_utl_3'], axis = 1)
data = data.drop(['fan_utl_4'], axis = 1)
data = data.drop(['fan_utl_5'], axis = 1)
data = data.drop(['fan_utl_6'], axis = 1)
data = data.drop(['tv_kamper_vist'], axis = 1)


# In[428]:


# Remove variables 17-24 that follow the second conditional question
data = data.drop(['medlem_no'], axis = 1)
data = data.drop(['fan_no_1'], axis = 1)
data = data.drop(['fan_no_2'], axis = 1)
data = data.drop(['fan_no_3'], axis = 1)
data = data.drop(['fan_no_4'], axis = 1)
data = data.drop(['fan_no_5'], axis = 1)
data = data.drop(['fan_no_6'], axis = 1)
data = data.drop(['kamper_no_sett'], axis = 1)


# In[429]:


# Remove variable 40 that follow the third conditional question
data = data.drop(['piratstrøm_antall'], axis = 1)


# In[430]:


# How many blank values are in variable 25 and 26?
print(data['wtp_utl'].value_counts())
print(data['wtp_no'].value_counts())


# In[431]:


# Remove variables 25 and 26 (WTP)
data = data.drop(['wtp_utl'], axis = 1)
data = data.drop(['wtp_no'], axis = 1)


# In[432]:


for col in data:
    print(data[col].unique())


# In[433]:


# How many blank values are in variable 41-43?
print(data['persgevinst'].value_counts())
print(data['persrisiko_1'].value_counts())
print(data['persrisiko_2'].value_counts())


# In[434]:


# All blank values in variables 41-43 will be converted to 4 ("verken eller")
data['persgevinst'] = data['persgevinst'].replace([' '], '5')
data['persrisiko_1'] = data['persrisiko_1'].replace([' '], '5')
data['persrisiko_2'] = data['persrisiko_2'].replace([' '], '5')


# In[435]:


# Variables 41-43 converted to integers
data = data.astype({'persgevinst':'int'})
data = data.astype({'persrisiko_1':'int'})
data = data.astype({'persrisiko_2':'int'})


# In[436]:


# Visualize dataset to see if all variables are integers and don't contain blank values
for col in data:
    print(data[col].unique())


# In[437]:


# We create dummy variables for variable 28
# Yes = 1, No = 2, Idk = 3
# Convert 2 & 3 to 0
data['pirat_selv'] = data['pirat_selv'].replace([2], 0)
data['pirat_selv'] = data['pirat_selv'].replace([3], 0)


# In[438]:


# Visualize the value counts in 'pirat_selv'
print(data['pirat_selv'].value_counts())


# In[439]:


# How many entries and variables do we have know?
data.shape


# In[440]:


# Get a list of all the variables after cleaning the data
data.info()


# # Analysis
# ## Correlation

# In[441]:


# Histograms
import matplotlib.pyplot as plt

data.hist(bins = 30, figsize = (30,30))
plt.show()


# In[442]:


# Correlation matrix
data.corr().style.background_gradient('coolwarm').set_precision(3)


# In[443]:


# Absolute correlation values in a list
corr_matrix = abs(data.corr())
corr_matrix['pirat_selv'].sort_values(ascending = False)


# ## Exploratory Logistic Regression

# In[444]:


# Split dependent and independent variables
X = data[['perskontroll_1','intensjon_1','perskontroll_2','intensjon_3',
          'interesse_4','intensjon_2','kjønn','pirat_venner','interesse_5','holdning_piracy_1','holdning_piracy_3',
          'favorittlag_utl','interesse_1','holdning_piracy_2','norm_personlig_1','interesse_3','rettferdiggjør_1',
          'norm_sosial_1','norm_personlig_2','norm_sosial_3','wtp_PL','tv_kamper_utl_sett','abonnement_PL3',
          'abonnement_PL1','lønn_PL1','lønn_PL2','favorittlag_no','lønn_PL3','norm_sosial_2','persgevinst',
          'abonnement_PL2','rettferdiggjør_2','norm_personlig_3','abonnerer_PL_kanal','interesse_6','persrisiko_1',
          'interesse_2','persrisiko_2','inntekt']]
y = data['pirat_selv']


# In[445]:


# Import necessary libraries and 
import pandas as pd
import statsmodels.api as sm
import numpy as np

# Standardize the independent variables
X_standardized = (X - X.mean()) / X.std()

# Add an intercept term for the logistic regression model
X_standardized['intercept'] = 1

# Fit the LASSO logistic regression model
# Set alpha to control the regularization strength, a larger value will result in more regularization
alpha = 0.1

glm_model = sm.GLM(
    y,
    X_standardized,
    family=sm.families.Binomial(link=sm.families.links.logit()),
)

result = glm_model.fit_regularized(alpha=alpha, L1_wt=1)

# Filter out the independent variables with non-zero coefficients
selected_variables = X_standardized.columns[np.abs(result.params) > 1e-5]

if len(selected_variables) > 0:
    # Refit the model using only the selected variables
    X_standardized_selected = X_standardized[selected_variables]

    logit_model_selected = sm.Logit(y, X_standardized_selected)
    result_selected = logit_model_selected.fit()

    # Print the summary of the refitted model
    print(result_selected.summary())
else:
    print("Error")


# ## Predictive Logistic Regression

# In[446]:


# Import necessary libraries
from sklearn.metrics import confusion_matrix, plot_confusion_matrix
import matplotlib.pyplot as plt
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split

# Split into training and testing set
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# Create the logistic regression model
logistic_regression_model = LogisticRegression()

# Train the model with your training data
logistic_regression_model.fit(X_train, y_train)

y_pred = logistic_regression_model.predict(X_test)
cm = confusion_matrix(y_test, y_pred)

# Plot the confusion matrix
plot_confusion_matrix(logistic_regression_model, X_test, y_test, cmap=plt.cm.Blues)
plt.title("Confusion Matrix")
plt.show()


# In[447]:


# Accuracy of the model
accuracy = accuracy_score(y_test, y_pred)
print('Accuracy: {:.2f}'.format(accuracy))


# In[448]:


# Import and install necessary libraries
get_ipython().system('pip install tabulate')

from sklearn.metrics import classification_report
from tabulate import tabulate

report = classification_report(y_test, y_pred, output_dict=True)

# Create a pandas DataFrame from the classification report dictionary
import pandas as pd
report_df = pd.DataFrame(report).transpose()

# Set display options for the DataFrame
pd.set_option('display.precision', 2)
pd.set_option('display.float_format', '{:.2f}'.format)

# Print the report as a formatted table using tabulate
print(tabulate(report_df, headers='keys', tablefmt='psql', numalign='right'))


# ## Naive Bayes

# In[449]:


# Import and install necessary libraries
get_ipython().system('pip install matplotlib seaborn yellowbrick')
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.metrics import ConfusionMatrixDisplay
from yellowbrick.classifier import ClassificationReport
from sklearn.model_selection import train_test_split
from sklearn.naive_bayes import GaussianNB
from sklearn.metrics import classification_report, confusion_matrix, accuracy_score


# In[450]:


# x-values are independent variables, y-value is the dependent variable
x_NB = data[['perskontroll_1','intensjon_1','perskontroll_2','intensjon_3',
          'interesse_4','intensjon_2','kjønn','pirat_venner','interesse_5','holdning_piracy_1','holdning_piracy_3',
          'favorittlag_utl','interesse_1','holdning_piracy_2','norm_personlig_1','interesse_3','rettferdiggjør_1',
          'norm_sosial_1','norm_personlig_2','norm_sosial_3','wtp_PL','tv_kamper_utl_sett','abonnement_PL3',
          'abonnement_PL1','lønn_PL1','lønn_PL2','favorittlag_no','lønn_PL3','norm_sosial_2','persgevinst',
          'abonnement_PL2','rettferdiggjør_2','norm_personlig_3','abonnerer_PL_kanal','interesse_6','persrisiko_1',
          'interesse_2','persrisiko_2','inntekt']]
y_NB = data['pirat_selv']


# In[451]:


# Split into train and test sets
x_NB_train, x_NB_test, y_NB_train, y_NB_test = train_test_split(x_NB, y_NB, test_size=0.2, random_state=42)


# In[452]:


# Create the model and fit the data
gnb = GaussianNB()
gnb.fit(x_NB_train, y_NB_train)


# In[453]:


# Make predictions on the test set
y_pred = gnb.predict(x_NB_test)


# In[454]:


# Confusion matrix
cm = confusion_matrix(y_NB_test, y_pred)
cm_display = ConfusionMatrixDisplay(cm).plot(cmap="Blues")
plt.title('Confusion Matrix')
plt.show()


# In[455]:


# Model accuracy
accuracy = accuracy_score(y_NB_test, y_pred)
print('Accuracy: {:.2f}'.format(accuracy))


# In[456]:


# Classification report
def plot_classification_report(y_true, y_pred, classifier):
    visualizer = ClassificationReport(classifier, support=True)
    visualizer.fit(x_NB_train, y_NB_train)  
    visualizer.score(x_NB_test, y_NB_test) 
    visualizer.show()                 
plot_classification_report(y_NB_test, y_pred, gnb)               


# In[457]:


# Import necessary library
from sklearn.model_selection import cross_val_score

# Cross-validation scores
scores = cross_val_score(gnb, x_NB, y_NB, cv=5)
print(scores)


# In[458]:


# Mean accuracy and standard deviation of cross-validation
mean_score = np.mean(scores)
std_score = np.std(scores)

print("Mean accuracy: {:.2f}%".format(mean_score * 100))
print("Standard deviation: {:.2f}%".format(std_score * 100))


# ## Decision Tree Classifier

# In[459]:


# Import necessary libraries
from sklearn.tree import DecisionTreeClassifier, plot_tree
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score


# In[460]:


# Split dependent and independent variables
x = data.drop('pirat_selv', axis=1) 
y = data['pirat_selv']


# In[461]:


# Split into training and test set
x_train, x_test, y_train, y_test = train_test_split(x, y, test_size=0.3, random_state=42)


# In[462]:


# Create decision tree classifier model and fit the model
clf = DecisionTreeClassifier()
clf.fit(x_train, y_train)


# In[463]:


# Create prediction of test set
y_pred = clf.predict(x_test)


# In[464]:


# Model accuracy
accuracy = accuracy_score(y_test, y_pred)
print('Accuracy:', accuracy)


# In[465]:


# Create and visualize the plot
plt.figure(figsize=(20, 10))
plot_tree(
    clf,
    feature_names=x_train.columns,  
    class_names=['Not piracy', 'Piracy'],  
    filled=True,  
    rounded=True,  
    fontsize=10,  
)
plt.show()


# In[466]:


# Get the feature importances
importances = clf.feature_importances_

features = feature_names

# Combine the features and importances into a single list and sort by importance
sorted_features_importances = sorted(zip(features, importances), key=lambda x: x[1], reverse=True)

# Print the sorted feature importances
for feature, importance in sorted_features_importances:
    print(f"{feature}: {importance}")


# ## K-Folds Cross-Validation

# In[467]:


import numpy as np
import pandas as pd
from sklearn.model_selection import KFold
from sklearn.tree import DecisionTreeClassifier
from sklearn.metrics import accuracy_score

# Separate features (X) and target (y)
X = data.drop('pirat_selv', axis=1).values  # Replace 'target' with the name of the target column in your dataset
y = data['pirat_selv'].values  # Replace 'target' with the name of the target column in your dataset
feature_names = data.drop('pirat_selv', axis=1).columns  # Replace 'target' with the name of the target column in your dataset

# Choose the number of folds (k)
k = 5

# Initialize the decision tree model
clf = DecisionTreeClassifier()

# Initialize the KFold object with k splits
kf = KFold(n_splits=k, shuffle=True, random_state=42)

# Initialize lists to store test accuracies and feature importances for each fold
test_accuracies = []
feature_importances = []

# Perform k-fold cross-validation
for train_index, test_index in kf.split(X):
    X_train, X_test = X[train_index], X[test_index]
    y_train, y_test = y[train_index], y[test_index]

    # Train the decision tree model on the training set
    clf.fit(X_train, y_train)

    # Evaluate the model's performance on the testing set and calculate the test accuracy
    y_pred = clf.predict(X_test)
    test_accuracy = accuracy_score(y_test, y_pred)

    # Store the test accuracy and feature importances for this fold
    test_accuracies.append(test_accuracy)
    feature_importances.append(clf.feature_importances_)

# Calculate the average test accuracy
mean_test_accuracy = np.mean(test_accuracies)

print(f"Test accuracies: {test_accuracies}")
print(f"Mean test accuracy: {mean_test_accuracy}")

# Calculate the average feature importances across all k folds
mean_feature_importances = np.mean(feature_importances, axis=0)

# Create a pandas DataFrame mapping feature names to their importances
importance_df = pd.DataFrame({'Feature': feature_names, 'Importance': mean_feature_importances})

# Sort the DataFrame by descending importance values
importance_df = importance_df.sort_values(by='Importance', ascending=False)

# Display the DataFrame
print("\nMean feature importances:")
print(importance_df)


# In[ ]:




